      subroutine calcpar(n,uuh,vvh,pvh)
C                        i  i   i   o
********************************************************************************
*                                                                              *
*     Computation of several boundary layer parameters needed for the          *
*     dispersion calculation and calculation of dry deposition velocities.     *
*     All parameters are calculated over the entire grid.                      *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     21 May 1995                                                              *
*                                                                              *
* ------------------------------------------------------------------           *
*     Petra Seibert, Feb 2000:                                                 *
*     convection scheme:                                                       *
*     new variables in call to richardson                                      *
*                                                                              *
********************************************************************************
*  Changes, Bernd C. Krueger, Feb. 2001:
*        Variables tth and qvh (on eta coordinates) in common block
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* n                  temporal index for meteorological fields (1 to 3)         *
*                                                                              *
* Constants:                                                                   *
*                                                                              *
*                                                                              *
* Functions:                                                                   *
* scalev             computation of ustar                                      *
* obukhov            computatio of Obukhov length                              *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'

      integer n,ix,jy,i,kz,lz,kzmin
      real ttlev(nuvzmax),qvlev(nuvzmax),obukhov,scalev,ol,hmixplus
      real ulev(nuvzmax),vlev(nuvzmax),ew,rh,vd(maxspec),subsceff,ylat
      real altmin,tvold,pold,zold,pint,tv,zlev(nuvzmax),const
      real uuh(0:nxmax-1,0:nymax-1,nuvzmax)
      real vvh(0:nxmax-1,0:nymax-1,nuvzmax)
      real pvh(0:nxmax-1,0:nymax-1,nuvzmax)
      parameter(const=r_air/ga)


C Loop over entire grid
***********************

      do 10 jy=0,ny-1

C Set minimum height for tropopause
***********************************

        ylat=ylat0+float(jy)*dy
        if ((ylat.ge.-20.).and.(ylat.le.20.)) then
          altmin = 5000.
        else
          if ((ylat.gt.20.).and.(ylat.lt.40.)) then
            altmin=2500.+(40.-ylat)*125.
          else if ((ylat.gt.-40.).and.(ylat.lt.-20.)) then
            altmin=2500.+(40.+ylat)*125.
          else
            altmin=2500.
          endif
        endif

        do 10 ix=0,nx-1

C 1) Calculation of friction velocity
*************************************

          ustar(ix,jy,1,n)=scalev(ps(ix,jy,1,n),tt2(ix,jy,1,n),
     +    td2(ix,jy,1,n),surfstr(ix,jy,1,n))
          if (ustar(ix,jy,1,n).le.1.e-8) ustar(ix,jy,1,n)=1.e-8

C 2) Calculation of inverse Obukhov length scale
************************************************

          ol=obukhov(ps(ix,jy,1,n),tt2(ix,jy,1,n),td2(ix,jy,1,n),
     +    tth(ix,jy,2,n),ustar(ix,jy,1,n),sshf(ix,jy,1,n),akm,bkm)
          if (ol.ne.0.) then
            oli(ix,jy,1,n)=1./ol
          else
            oli(ix,jy,1,n)=99999.
          endif


C 3) Calculation of convective velocity scale and mixing height
***************************************************************

          do 20 i=1,nuvz
            ulev(i)=uuh(ix,jy,i)
            vlev(i)=vvh(ix,jy,i)
            ttlev(i)=tth(ix,jy,i,n)
20          qvlev(i)=qvh(ix,jy,i,n)

          call richardson(ps(ix,jy,1,n),ustar(ix,jy,1,n),ttlev,qvlev,
     +    ulev,vlev,nuvz,akz,bkz,sshf(ix,jy,1,n),tt2(ix,jy,1,n),
     +    td2(ix,jy,1,n),hmix(ix,jy,1,n),wstar(ix,jy,1,n),hmixplus)

          if(lsubgrid.eq.1) then
            subsceff=min(excessoro(ix,jy),hmixplus)
          else
            subsceff=0
          endif
*
* CALCULATE HMIX EXCESS ACCORDING TO SUBGRIDSCALE VARIABILITY AND STABILITY
*
          hmix(ix,jy,1,n)=hmix(ix,jy,1,n)+subsceff
          hmix(ix,jy,1,n)=max(hmixmin,hmix(ix,jy,1,n)) ! set minimum PBL height
          hmix(ix,jy,1,n)=min(hmixmax,hmix(ix,jy,1,n)) ! set maximum PBL height

C 4) Calculation of dry deposition velocities
*********************************************

          if (DRYDEP) then
            z0(4)=0.016*ustar(ix,jy,1,n)*ustar(ix,jy,1,n)/ga
            z0(9)=0.016*ustar(ix,jy,1,n)*ustar(ix,jy,1,n)/ga

C Calculate relative humidity at surface
****************************************
            rh=ew(td2(ix,jy,1,n))/ew(tt2(ix,jy,1,n))

            call getvdep(n,ix,jy,ustar(ix,jy,1,n),
     +      tt2(ix,jy,1,n),ps(ix,jy,1,n),1./oli(ix,jy,1,n),
     +      ssr(ix,jy,1,n),rh,lsprec(ix,jy,1,n)+convprec(ix,jy,1,n),vd)

            do 30 i=1,nspec
30            vdep(ix,jy,i,n)=vd(i)

          endif

*******************************************************
C Calculate height of thermal tropopause (Hoinka, 1997)
*******************************************************

C 1) Calculate altitudes of ECMWF model levels
**********************************************

          tvold=tt2(ix,jy,1,n)*(1.+0.378*ew(td2(ix,jy,1,n))/
     +                                   ps(ix,jy,1,n))
          pold=ps(ix,jy,1,n)
          zold=0.
          do 40 kz=2,nuvz
            pint=akz(kz)+bkz(kz)*ps(ix,jy,1,n)  ! pressure on model layers
            tv=tth(ix,jy,kz,n)*(1.+0.608*qvh(ix,jy,kz,n))

            if (abs(tv-tvold).gt.0.2) then
             zlev(kz)=zold+const*log(pold/pint)*(tv-tvold)/log(tv/tvold)
            else
              zlev(kz)=zold+const*log(pold/pint)*tv
            endif
            tvold=tv
            pold=pint
            zold=zlev(kz)
40          continue

C 2) Define a minimum level kzmin, from which upward the tropopause is
C    searched for. This is to avoid inversions in the lower troposphere
C    to be identified as the tropopause
*************************************************************************

          do 44 kz=1,nuvz
            if (zlev(kz).ge.altmin) then
              kzmin=kz
              goto 45
            endif
44          continue
45        continue

C 3) Search for first stable layer above minimum height that fulfills the
C    thermal tropopause criterion
*************************************************************************

          do 50 kz=kzmin,nuvz
            do 60 lz=kz+1,nuvz
              if ((zlev(lz)-zlev(kz)).gt.2000.) then
                if (((tth(ix,jy,kz,n)-tth(ix,jy,lz,n))/
     +          (zlev(lz)-zlev(kz))).lt.0.002) then
                  tropopause(ix,jy,1,n)=zlev(kz)
                  goto 51
                endif
                goto 50
              endif
60            continue
50          continue
51        continue


10        continue


C Calculation of potential vorticity on 3-d grid, if plume trajectory mode is used
**********************************************************************************

      if ((iout.eq.4).or.(iout.eq.5).or.(mdomainfill.eq.2)) then
        call calcpv(n,uuh,vvh,pvh)
      endif


      end
