      subroutine calcpar_nests(n,uuhn,vvhn,pvhn)
C                              i  i    i    o
********************************************************************************
*                                                                              *
*     Computation of several boundary layer parameters needed for the          *
*     dispersion calculation and calculation of dry deposition velocities.     *
*     All parameters are calculated over the entire grid.                      *
*     This routine is similar to calcpar, but is used for the nested grids.    *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     8 February 1999                                                          *
*                                                                              *
* ------------------------------------------------------------------           *
*     Petra Seibert, Feb 2000:                                                 *
*     convection scheme:                                                       *
*     new variables in call to richardson                                      *
*                                                                              *
********************************************************************************
*  Changes, Bernd C. Krueger, Feb. 2001:
*        Variables tth and qvh (on eta coordinates) in common block
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* n                  temporal index for meteorological fields (1 to 3)         *
*                                                                              *
* Constants:                                                                   *
*                                                                              *
*                                                                              *
* Functions:                                                                   *
* scalev             computation of ustar                                      *
* obukhov            computatio of Obukhov length                              *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'

      integer n,ix,jy,i,l,kz,lz,kzmin
      real ttlev(nuvzmax),qvlev(nuvzmax),obukhov,scalev,ol,hmixplus
      real ulev(nuvzmax),vlev(nuvzmax),ew,rh,vd(maxspec),subsceff,ylat
      real altmin,tvold,pold,zold,pint,tv,zlev(nuvzmax),const
      real uuhn(0:nxmaxn-1,0:nymaxn-1,nuvzmax,maxnests)
      real vvhn(0:nxmaxn-1,0:nymaxn-1,nuvzmax,maxnests)
      real pvhn(0:nxmaxn-1,0:nymaxn-1,nuvzmax,maxnests)
      parameter(const=r_air/ga)


C Loop over all nests
*********************

      do 80 l=1,numbnests

C Loop over entire grid
***********************

      do 10 jy=0,nyn(l)-1

C Set minimum height for tropopause
***********************************

        ylat=ylat0n(l)+float(jy)*dyn(l)
        if ((ylat.ge.-20.).and.(ylat.le.20.)) then
          altmin = 5000.
        else
          if ((ylat.gt.20.).and.(ylat.lt.40.)) then
            altmin=2500.+(40.-ylat)*125.
          else if ((ylat.gt.-40.).and.(ylat.lt.-20.)) then
            altmin=2500.+(40.+ylat)*125.
          else
            altmin=2500.
          endif
        endif

        do 10 ix=0,nxn(l)-1

C 1) Calculation of friction velocity
*************************************

          ustarn(ix,jy,1,n,l)=scalev(psn(ix,jy,1,n,l),tt2n(ix,jy,1,n,l),
     +    td2n(ix,jy,1,n,l),surfstrn(ix,jy,1,n,l))

C 2) Calculation of inverse Obukhov length scale
************************************************

          ol=obukhov(psn(ix,jy,1,n,l),tt2n(ix,jy,1,n,l),
     +    td2n(ix,jy,1,n,l),tthn(ix,jy,2,n,l),ustarn(ix,jy,1,n,l),
     +    sshfn(ix,jy,1,n,l),akm,bkm)
          if (ol.ne.0.) then
            olin(ix,jy,1,n,l)=1./ol
          else
            olin(ix,jy,1,n,l)=99999.
          endif


C 3) Calculation of convective velocity scale and mixing height
***************************************************************

          do 20 i=1,nuvz
            ulev(i)=uuhn(ix,jy,i,l)
            vlev(i)=vvhn(ix,jy,i,l)
            ttlev(i)=tthn(ix,jy,i,n,l)
20          qvlev(i)=qvhn(ix,jy,i,n,l)

          call richardson(psn(ix,jy,1,n,l),ustarn(ix,jy,1,n,l),ttlev,
     +    qvlev,ulev,vlev,nuvz,akz,bkz,sshfn(ix,jy,1,n,l),
     +    tt2n(ix,jy,1,n,l),td2n(ix,jy,1,n,l),hmixn(ix,jy,1,n,l),
     +    wstarn(ix,jy,1,n,l),hmixplus)

          if(lsubgrid.eq.1) then
            subsceff=min(excessoron(ix,jy,l),hmixplus)
          else
            subsceff=0
          endif
*
* CALCULATE HMIX EXCESS ACCORDING TO SUBGRIDSCALE VARIABILITY AND STABILITY
*
          hmixn(ix,jy,1,n,l)=hmixn(ix,jy,1,n,l)+subsceff
          hmixn(ix,jy,1,n,l)=max(hmixmin,hmixn(ix,jy,1,n,l)) ! minim PBL height
          hmixn(ix,jy,1,n,l)=min(hmixmax,hmixn(ix,jy,1,n,l)) ! maxim PBL height


C 4) Calculation of dry deposition velocities
*********************************************

          if (DRYDEP) then
            z0(4)=0.016*ustarn(ix,jy,1,n,l)*ustarn(ix,jy,1,n,l)/ga
            z0(9)=0.016*ustarn(ix,jy,1,n,l)*ustarn(ix,jy,1,n,l)/ga

C Calculate relative humidity at surface
****************************************
            rh=ew(td2n(ix,jy,1,n,l))/ew(tt2n(ix,jy,1,n,l))

            call getvdep(n,ix,jy,ustarn(ix,jy,1,n,l),
     +      tt2n(ix,jy,1,n,l),psn(ix,jy,1,n,l),1./olin(ix,jy,1,n,l),
     +      ssrn(ix,jy,1,n,l),rh,lsprecn(ix,jy,1,n,l)+
     +      convprecn(ix,jy,1,n,l),vd)

            do 30 i=1,nspec
30            vdepn(ix,jy,i,n,l)=vd(i)

          endif

*******************************************************
C Calculate height of thermal tropopause (Hoinka, 1997)
*******************************************************

C 1) Calculate altitudes of ECMWF model levels
**********************************************

          tvold=tt2n(ix,jy,1,n,l)*(1.+0.378*ew(td2n(ix,jy,1,n,l))/
     +                                   psn(ix,jy,1,n,l))
          pold=psn(ix,jy,1,n,l)
          zold=0.
          do 40 kz=2,nuvz
            pint=akz(kz)+bkz(kz)*psn(ix,jy,1,n,l)  ! pressure on model layers
            tv=tthn(ix,jy,kz,n,l)*(1.+0.608*qvhn(ix,jy,kz,n,l))

            if (abs(tv-tvold).gt.0.2) then
             zlev(kz)=zold+const*log(pold/pint)*(tv-tvold)/log(tv/tvold)
            else
              zlev(kz)=zold+const*log(pold/pint)*tv
            endif
            tvold=tv
            pold=pint
            zold=zlev(kz)
40          continue

C 2) Define a minimum level kzmin, from which upward the tropopause is
C    searched for. This is to avoid inversions in the lower troposphere
C    to be identified as the tropopause
*************************************************************************

          do 44 kz=1,nuvz
            if (zlev(kz).ge.altmin) then
              kzmin=kz
              goto 45
            endif
44          continue
45        continue

C 3) Search for first stable layer above minimum height that fulfills the
C    thermal tropopause criterion
*************************************************************************

          do 50 kz=kzmin,nuvz
            do 60 lz=kz+1,nuvz
              if ((zlev(lz)-zlev(kz)).gt.2000.) then
                if (((tthn(ix,jy,kz,n,l)-tthn(ix,jy,lz,n,l))/
     +          (zlev(lz)-zlev(kz))).lt.0.002) then
                  tropopausen(ix,jy,1,n,l)=zlev(kz)
                  goto 51
                endif
                goto 50
              endif
60            continue
50          continue
51        continue


10        continue


C Calculation of potential vorticity on 3-d grid, if plume trajectory mode is used
**********************************************************************************

        if ((iout.eq.4).or.(iout.eq.5)) then
          call calcpv_nests(l,n,uuhn,vvhn,pvhn)
        endif

80      continue


      end
