      subroutine interpol_all(itime,xt,yt,zt,dxdtprof,dydtprof,
     +dzdtprof,usigprof,vsigprof,wsigprof,rhoprof,rhogradprof,ust,
     +wst,ol)
C                               i   i  i  i     o        o
C        o        o        o        o        o         o       o
C      o  o
********************************************************************************
*                                                                              *
*  This subroutine interpolates everything that is needed for calculating the  *
*  dispersion.                                                                 *
*                                                                              *
*    Author: A. Stohl                                                          *
*                                                                              *
*    16 December 1997                                                          *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* itime [s]          current temporal position                                 *
* memtime(3) [s]     times of the wind fields in memory                        *
* xt,yt,zt           coordinates position for which wind data shall be calculat*
*                                                                              *
* Constants:                                                                   *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'
      include 'includeinterpol'

      integer itime
      real xt,yt,zt
      real dxdtprof(nzmax),dydtprof(nzmax),dzdtprof(nzmax)
      real usigprof(nzmax),vsigprof(nzmax),wsigprof(nzmax)
      real rhoprof(nzmax),rhogradprof(nzmax),ust,wst,ol

C Auxiliary variables needed for interpolation
      real ust1(2),wst1(2),oli1(2),oliaux
      real y1(2),y2(2),y3(2),rho1(2),rhograd1(2),us(8),vs(8),ws(8)
      integer i,m,n,nn,indexh


*********************************************
C Multilinear interpolation in time and space
*********************************************

C Determine the lower left corner and its distance to the current position
**************************************************************************

      ddx=xt-float(ix)                     
      ddy=yt-float(jy)
      rddx=1.-ddx
      rddy=1.-ddy
      p1=rddx*rddy
      p2=ddx*rddy
      p3=rddx*ddy
      p4=ddx*ddy

C Calculate variables for time interpolation
********************************************

      dt1=float(itime-memtime(1))
      dt2=float(memtime(2)-itime)
      dtt=1./(dt1+dt2)


******************************************
C 1. Interpolate u*, w* and Obukhov length
******************************************

C a) Bilinear horizontal interpolation

      do 80 m=1,2
        indexh=memind(m)

        ust1(m)=p1*ustar(ix ,jy ,1,indexh)
     +        + p2*ustar(ixp,jy ,1,indexh)
     +        + p3*ustar(ix ,jyp,1,indexh)
     +        + p4*ustar(ixp,jyp,1,indexh)
        wst1(m)=p1*wstar(ix ,jy ,1,indexh)
     +        + p2*wstar(ixp,jy ,1,indexh)
     +        + p3*wstar(ix ,jyp,1,indexh)
     +        + p4*wstar(ixp,jyp,1,indexh)
80      oli1(m)=p1*oli(ix ,jy ,1,indexh)
     +        + p2*oli(ixp,jy ,1,indexh)
     +        + p3*oli(ix ,jyp,1,indexh)
     +        + p4*oli(ixp,jyp,1,indexh)

C b) Temporal interpolation

      ust=(ust1(1)*dt2+ust1(2)*dt1)*dtt
      wst=(wst1(1)*dt2+wst1(2)*dt1)*dtt
      oliaux=(oli1(1)*dt2+oli1(2)*dt1)*dtt

      if (oliaux.ne.0.) then
        ol=1./oliaux
      else
        ol=99999.
      endif


******************************************************
C 2. Interpolate vertical profiles of u,v,w,rho,drhodz
******************************************************


C Determine the level below the current position
************************************************

      do 5 i=2,nz
        if (height(i).gt.zt) then
          indz=i-1
          indzp=i
          goto 6
        endif
5       continue
6     continue

***************************************
C 1.) Bilinear horizontal interpolation
C 2.) Temporal interpolation (linear)
***************************************

C Loop over 2 time steps and indz levels
****************************************

      do 10 n=indz,indzp
        do 20 m=1,2
          indexh=memind(m)
          nn=(m-1)*4
          if (ngrid.lt.0) then
            y1(m)=p1*uupol(ix ,jy ,n,indexh)
     +           +p2*uupol(ixp,jy ,n,indexh)
     +           +p3*uupol(ix ,jyp,n,indexh)
     +           +p4*uupol(ixp,jyp,n,indexh)
            y2(m)=p1*vvpol(ix ,jy ,n,indexh)
     +           +p2*vvpol(ixp,jy ,n,indexh)
     +           +p3*vvpol(ix ,jyp,n,indexh)
     +           +p4*vvpol(ixp,jyp,n,indexh)
            us(nn+1)=uupol(ix ,jy ,n,indexh)
            us(nn+2)=uupol(ixp,jy ,n,indexh)
            us(nn+3)=uupol(ix ,jyp,n,indexh)
            us(nn+4)=uupol(ixp,jyp,n,indexh)
            vs(nn+1)=vvpol(ix ,jy ,n,indexh)
            vs(nn+2)=vvpol(ixp,jy ,n,indexh)
            vs(nn+3)=vvpol(ix ,jyp,n,indexh)
            vs(nn+4)=vvpol(ixp,jyp,n,indexh)
          else
            y1(m)=p1*uu(ix ,jy ,n,indexh)
     +           +p2*uu(ixp,jy ,n,indexh)
     +           +p3*uu(ix ,jyp,n,indexh)
     +           +p4*uu(ixp,jyp,n,indexh)
            y2(m)=p1*vv(ix ,jy ,n,indexh)
     +           +p2*vv(ixp,jy ,n,indexh)
     +           +p3*vv(ix ,jyp,n,indexh)
     +           +p4*vv(ixp,jyp,n,indexh)
            us(nn+1)=uu(ix ,jy ,n,indexh)
            us(nn+2)=uu(ixp,jy ,n,indexh)
            us(nn+3)=uu(ix ,jyp,n,indexh)
            us(nn+4)=uu(ixp,jyp,n,indexh)
            vs(nn+1)=vv(ix ,jy ,n,indexh)
            vs(nn+2)=vv(ixp,jy ,n,indexh)
            vs(nn+3)=vv(ix ,jyp,n,indexh)
            vs(nn+4)=vv(ixp,jyp,n,indexh)
          endif
          y3(m)=p1*ww(ix ,jy ,n,indexh)
     +         +p2*ww(ixp,jy ,n,indexh)
     +         +p3*ww(ix ,jyp,n,indexh)
     +         +p4*ww(ixp,jyp,n,indexh)
          rhograd1(m)=p1*drhodz(ix ,jy ,n,indexh)
     +               +p2*drhodz(ixp,jy ,n,indexh)
     +               +p3*drhodz(ix ,jyp,n,indexh)
     +               +p4*drhodz(ixp,jyp,n,indexh)
          rho1(m)=p1*rho(ix ,jy ,n,indexh)
     +           +p2*rho(ixp,jy ,n,indexh)
     +           +p3*rho(ix ,jyp,n,indexh)
     +           +p4*rho(ixp,jyp,n,indexh)
          ws(nn+1)=ww(ix ,jy ,n,indexh)
          ws(nn+2)=ww(ixp,jy ,n,indexh)
          ws(nn+3)=ww(ix ,jyp,n,indexh)
          ws(nn+4)=ww(ixp,jyp,n,indexh)
20        continue
        dxdtprof(n)=(y1(1)*dt2+y1(2)*dt1)*dtt
        dydtprof(n)=(y2(1)*dt2+y2(2)*dt1)*dtt
        dzdtprof(n)=(y3(1)*dt2+y3(2)*dt1)*dtt
        rhoprof(n)=(rho1(1)*dt2+rho1(2)*dt1)*dtt
        rhogradprof(n)=(rhograd1(1)*dt2+rhograd1(2)*dt1)*dtt
        call sigma(us,usigprof(n),8)
        call sigma(vs,vsigprof(n),8)
        call sigma(ws,wsigprof(n),8)
10      indzindicator(n)=.false.


      end
