      subroutine interpol_all_nests(itime,xt,yt,zt,dxdtprof,
     +dydtprof,dzdtprof,usigprof,vsigprof,wsigprof,rhoprof,rhogradprof,
     +ust,wst,ol)
C                                     i   i  i  i     o
C        o        o        o        o        o        o         o
C      o   o  o
********************************************************************************
*                                                                              *
*  This subroutine interpolates everything that is needed for calculating the  *
*  dispersion.                                                                 *
*  Version for interpolating nested grids.                                     *
*                                                                              *
*    Author: A. Stohl                                                          *
*                                                                              *
*    9 February 1999                                                           *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* itime [s]          current temporal position                                 *
* memtime(3) [s]     times of the wind fields in memory                        *
* xt,yt,zt           coordinates position for which wind data shall be calculat*
*                                                                              *
* Constants:                                                                   *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'
      include 'includeinterpol'

      integer itime
      real xt,yt,zt
      real dxdtprof(nzmax),dydtprof(nzmax),dzdtprof(nzmax)
      real usigprof(nzmax),vsigprof(nzmax),wsigprof(nzmax)
      real rhoprof(nzmax),rhogradprof(nzmax),ust,wst,ol

C Auxiliary variables needed for interpolation
      real ust1(2),wst1(2),oli1(2),oliaux
      real y1(2),y2(2),y3(2),rho1(2),rhograd1(2),us(8),vs(8),ws(8)
      integer i,m,n,nn,indexh


*********************************************
C Multilinear interpolation in time and space
*********************************************

C Determine the lower left corner and its distance to the current position
**************************************************************************

      ddx=xt-float(ix)                     
      ddy=yt-float(jy)
      rddx=1.-ddx
      rddy=1.-ddy
      p1=rddx*rddy
      p2=ddx*rddy
      p3=rddx*ddy
      p4=ddx*ddy

C Calculate variables for time interpolation
********************************************

      dt1=float(itime-memtime(1))
      dt2=float(memtime(2)-itime)
      dtt=1./(dt1+dt2)


******************************************
C 1. Interpolate u*, w* and Obukhov length
******************************************

C a) Bilinear horizontal interpolation

      do 80 m=1,2
        indexh=memind(m)

        ust1(m)=p1*ustarn(ix ,jy ,1,indexh,ngrid)
     +        + p2*ustarn(ixp,jy ,1,indexh,ngrid)
     +        + p3*ustarn(ix ,jyp,1,indexh,ngrid)
     +        + p4*ustarn(ixp,jyp,1,indexh,ngrid)
        wst1(m)=p1*wstarn(ix ,jy ,1,indexh,ngrid)
     +        + p2*wstarn(ixp,jy ,1,indexh,ngrid)
     +        + p3*wstarn(ix ,jyp,1,indexh,ngrid)
     +        + p4*wstarn(ixp,jyp,1,indexh,ngrid)
80      oli1(m)=p1*olin(ix ,jy ,1,indexh,ngrid)
     +        + p2*olin(ixp,jy ,1,indexh,ngrid)
     +        + p3*olin(ix ,jyp,1,indexh,ngrid)
     +        + p4*olin(ixp,jyp,1,indexh,ngrid)

C b) Temporal interpolation

      ust=(ust1(1)*dt2+ust1(2)*dt1)*dtt
      wst=(wst1(1)*dt2+wst1(2)*dt1)*dtt
      oliaux=(oli1(1)*dt2+oli1(2)*dt1)*dtt

      if (oliaux.ne.0.) then
        ol=1./oliaux
      else
        ol=99999.
      endif


******************************************************
C 2. Interpolate vertical profiles of u,v,w,rho,drhodz
******************************************************


C Determine the level below the current position
************************************************

      do 5 i=2,nz
        if (height(i).gt.zt) then
          indz=i-1
          indzp=i
          goto 6
        endif
5       continue
6     continue

***************************************
C 1.) Bilinear horizontal interpolation
C 2.) Temporal interpolation (linear)
***************************************

C Loop over 2 time steps and indz levels
****************************************

      do 10 n=indz,indz+1
        do 20 m=1,2
          indexh=memind(m)
          nn=(m-1)*4
          y1(m)=p1*uun(ix ,jy ,n,indexh,ngrid)
     +         +p2*uun(ixp,jy ,n,indexh,ngrid)
     +         +p3*uun(ix ,jyp,n,indexh,ngrid)
     +         +p4*uun(ixp,jyp,n,indexh,ngrid)
          y2(m)=p1*vvn(ix ,jy ,n,indexh,ngrid)
     +         +p2*vvn(ixp,jy ,n,indexh,ngrid)
     +         +p3*vvn(ix ,jyp,n,indexh,ngrid)
     +         +p4*vvn(ixp,jyp,n,indexh,ngrid)
          y3(m)=p1*wwn(ix ,jy ,n,indexh,ngrid)
     +         +p2*wwn(ixp,jy ,n,indexh,ngrid)
     +         +p3*wwn(ix ,jyp,n,indexh,ngrid)
     +         +p4*wwn(ixp,jyp,n,indexh,ngrid)
          rhograd1(m)=p1*drhodzn(ix ,jy ,n,indexh,ngrid)
     +               +p2*drhodzn(ixp,jy ,n,indexh,ngrid)
     +               +p3*drhodzn(ix ,jyp,n,indexh,ngrid)
     +               +p4*drhodzn(ixp,jyp,n,indexh,ngrid)
          us(nn+1)=uun(ix ,jy ,n,indexh,ngrid)
          us(nn+2)=uun(ixp,jy ,n,indexh,ngrid)
          us(nn+3)=uun(ix ,jyp,n,indexh,ngrid)
          us(nn+4)=uun(ixp,jyp,n,indexh,ngrid)
          vs(nn+1)=vvn(ix ,jy ,n,indexh,ngrid)
          vs(nn+2)=vvn(ixp,jy ,n,indexh,ngrid)
          vs(nn+3)=vvn(ix ,jyp,n,indexh,ngrid)
          vs(nn+4)=vvn(ixp,jyp,n,indexh,ngrid)
          ws(nn+1)=wwn(ix ,jy ,n,indexh,ngrid)
          ws(nn+2)=wwn(ixp,jy ,n,indexh,ngrid)
          ws(nn+3)=wwn(ix ,jyp,n,indexh,ngrid)
          ws(nn+4)=wwn(ixp,jyp,n,indexh,ngrid)
          rho1(m)=p1*rhon(ix ,jy ,n,indexh,ngrid)
     +           +p2*rhon(ixp,jy ,n,indexh,ngrid)
     +           +p3*rhon(ix ,jyp,n,indexh,ngrid)
     +           +p4*rhon(ixp,jyp,n,indexh,ngrid)
20        continue
        dxdtprof(n)=(y1(1)*dt2+y1(2)*dt1)*dtt
        dydtprof(n)=(y2(1)*dt2+y2(2)*dt1)*dtt
        dzdtprof(n)=(y3(1)*dt2+y3(2)*dt1)*dtt
        rhoprof(n)=(rho1(1)*dt2+rho1(2)*dt1)*dtt
        rhogradprof(n)=(rhograd1(1)*dt2+rhograd1(2)*dt1)*dtt
        call sigma(us,usigprof(n),8)
        call sigma(vs,vsigprof(n),8)
        call sigma(ws,wsigprof(n),8)
10      indzindicator(n)=.false.

      end
