      subroutine interpol_misslev(n,dxdtprof,dydtprof,dzdtprof,
     +rhoprof,rhogradprof,usigprof,vsigprof,wsigprof)
C                                 i    o        o        o
C        o        o          o        o        o
********************************************************************************
*                                                                              *
*  This subroutine interpolates u,v,w, density and density gradients.          *
*                                                                              *
*    Author: A. Stohl                                                          *
*                                                                              *
*    16 December 1997                                                          *
*    Update: 2 March 1999                                                      *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* n                  level                                                     *
*                                                                              *
* Constants:                                                                   *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'
      include 'includeinterpol'

      real dxdtprof(nzmax),dydtprof(nzmax),dzdtprof(nzmax)
      real rhoprof(nzmax),rhogradprof(nzmax)
      real usigprof(nzmax),vsigprof(nzmax),wsigprof(nzmax)

C Auxiliary variables needed for interpolation
      real y1(2),y2(2),y3(2),rho1(2),rhograd1(2),us(8),vs(8),ws(8)
      integer m,n,nn,indexh


*********************************************
C Multilinear interpolation in time and space
*********************************************


***************************************
C 1.) Bilinear horizontal interpolation
C 2.) Temporal interpolation (linear)
***************************************

C Loop over 2 time steps
************************

      do 20 m=1,2
        indexh=memind(m)
        nn=(m-1)*4
        if (ngrid.lt.0) then
          y1(m)=p1*uupol(ix ,jy ,n,indexh)
     +         +p2*uupol(ixp,jy ,n,indexh)
     +         +p3*uupol(ix ,jyp,n,indexh)
     +         +p4*uupol(ixp,jyp,n,indexh)
          y2(m)=p1*vvpol(ix ,jy ,n,indexh)
     +         +p2*vvpol(ixp,jy ,n,indexh)
     +         +p3*vvpol(ix ,jyp,n,indexh)
     +         +p4*vvpol(ixp,jyp,n,indexh)
          us(nn+1)=uupol(ix ,jy ,n,indexh)
          us(nn+2)=uupol(ixp,jy ,n,indexh)
          us(nn+3)=uupol(ix ,jyp,n,indexh)
          us(nn+4)=uupol(ixp,jyp,n,indexh)
          vs(nn+1)=vvpol(ix ,jy ,n,indexh)
          vs(nn+2)=vvpol(ixp,jy ,n,indexh)
          vs(nn+3)=vvpol(ix ,jyp,n,indexh)
          vs(nn+4)=vvpol(ixp,jyp,n,indexh)
        else
          y1(m)=p1*uu(ix ,jy ,n,indexh)
     +         +p2*uu(ixp,jy ,n,indexh)
     +         +p3*uu(ix ,jyp,n,indexh)
     +         +p4*uu(ixp,jyp,n,indexh)
          y2(m)=p1*vv(ix ,jy ,n,indexh)
     +         +p2*vv(ixp,jy ,n,indexh)
     +         +p3*vv(ix ,jyp,n,indexh)
     +         +p4*vv(ixp,jyp,n,indexh)
          us(nn+1)=uu(ix ,jy ,n,indexh)
          us(nn+2)=uu(ixp,jy ,n,indexh)
          us(nn+3)=uu(ix ,jyp,n,indexh)
          us(nn+4)=uu(ixp,jyp,n,indexh)
          vs(nn+1)=vv(ix ,jy ,n,indexh)
          vs(nn+2)=vv(ixp,jy ,n,indexh)
          vs(nn+3)=vv(ix ,jyp,n,indexh)
          vs(nn+4)=vv(ixp,jyp,n,indexh)
        endif
        y3(m)=p1*ww(ix ,jy ,n,indexh)
     +       +p2*ww(ixp,jy ,n,indexh)
     +       +p3*ww(ix ,jyp,n,indexh)
     +       +p4*ww(ixp,jyp,n,indexh)
        rhograd1(m)=p1*drhodz(ix ,jy ,n,indexh)
     +             +p2*drhodz(ixp,jy ,n,indexh)
     +             +p3*drhodz(ix ,jyp,n,indexh)
     +             +p4*drhodz(ixp,jyp,n,indexh)
        rho1(m)=p1*rho(ix ,jy ,n,indexh)
     +         +p2*rho(ixp,jy ,n,indexh)
     +         +p3*rho(ix ,jyp,n,indexh)
     +         +p4*rho(ixp,jyp,n,indexh)
        ws(nn+1)=ww(ix ,jy ,n,indexh)
        ws(nn+2)=ww(ixp,jy ,n,indexh)
        ws(nn+3)=ww(ix ,jyp,n,indexh)
        ws(nn+4)=ww(ixp,jyp,n,indexh)
20      continue
      dxdtprof(n)=(y1(1)*dt2+y1(2)*dt1)*dtt
      dydtprof(n)=(y2(1)*dt2+y2(2)*dt1)*dtt
      dzdtprof(n)=(y3(1)*dt2+y3(2)*dt1)*dtt
      rhoprof(n)=(rho1(1)*dt2+rho1(2)*dt1)*dtt
      rhogradprof(n)=(rhograd1(1)*dt2+rhograd1(2)*dt1)*dtt
      indzindicator(n)=.false.


C Compute standard deviations
*****************************

      call sigma(us,usigprof(n),8)
      call sigma(vs,vsigprof(n),8)
      call sigma(ws,wsigprof(n),8)


      end
