      subroutine interpol_misslev_nests(n,dxdtprof,dydtprof,dzdtprof,
     +rhoprof,rhogradprof,usigprof,vsigprof,wsigprof)
C                                       i    o        o        o
C        o        o          o        o        o
********************************************************************************
*                                                                              *
*  This subroutine interpolates u,v,w, density and density gradients.          *
*                                                                              *
*    Author: A. Stohl                                                          *
*                                                                              *
*    16 December 1997                                                          *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* n                  level                                                     *
*                                                                              *
* Constants:                                                                   *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'
      include 'includeinterpol'

      real dxdtprof(nzmax),dydtprof(nzmax),dzdtprof(nzmax)
      real rhoprof(nzmax),rhogradprof(nzmax)
      real usigprof(nzmax),vsigprof(nzmax),wsigprof(nzmax)

C Auxiliary variables needed for interpolation
      real y1(2),y2(2),y3(2),rho1(2),rhograd1(2),us(8),vs(8),ws(8)
      integer m,n,nn,indexh


*********************************************
C Multilinear interpolation in time and space
*********************************************


***************************************
C 1.) Bilinear horizontal interpolation
C 2.) Temporal interpolation (linear)
***************************************

C Loop over 2 time steps
************************

      do 20 m=1,2
        indexh=memind(m)
        nn=(m-1)*4
        y1(m)=p1*uun(ix ,jy ,n,indexh,ngrid)
     +       +p2*uun(ixp,jy ,n,indexh,ngrid)
     +       +p3*uun(ix ,jyp,n,indexh,ngrid)
     +       +p4*uun(ixp,jyp,n,indexh,ngrid)
        y2(m)=p1*vvn(ix ,jy ,n,indexh,ngrid)
     +       +p2*vvn(ixp,jy ,n,indexh,ngrid)
     +       +p3*vvn(ix ,jyp,n,indexh,ngrid)
     +       +p4*vvn(ixp,jyp,n,indexh,ngrid)
        y3(m)=p1*wwn(ix ,jy ,n,indexh,ngrid)
     +       +p2*wwn(ixp,jy ,n,indexh,ngrid)
     +       +p3*wwn(ix ,jyp,n,indexh,ngrid)
     +       +p4*wwn(ixp,jyp,n,indexh,ngrid)
        rho1(m)=p1*rhon(ix ,jy ,n,indexh,ngrid)
     +         +p2*rhon(ixp,jy ,n,indexh,ngrid)
     +         +p3*rhon(ix ,jyp,n,indexh,ngrid)
     +         +p4*rhon(ixp,jyp,n,indexh,ngrid)
        rhograd1(m)=p1*drhodzn(ix ,jy ,n,indexh,ngrid)
     +             +p2*drhodzn(ixp,jy ,n,indexh,ngrid)
     +             +p3*drhodzn(ix ,jyp,n,indexh,ngrid)
     +             +p4*drhodzn(ixp,jyp,n,indexh,ngrid)
        us(nn+1)=uun(ix ,jy ,n,indexh,ngrid)
        us(nn+2)=uun(ixp,jy ,n,indexh,ngrid)
        us(nn+3)=uun(ix ,jyp,n,indexh,ngrid)
        us(nn+4)=uun(ixp,jyp,n,indexh,ngrid)
        vs(nn+1)=vvn(ix ,jy ,n,indexh,ngrid)
        vs(nn+2)=vvn(ixp,jy ,n,indexh,ngrid)
        vs(nn+3)=vvn(ix ,jyp,n,indexh,ngrid)
        vs(nn+4)=vvn(ixp,jyp,n,indexh,ngrid)
        ws(nn+1)=wwn(ix ,jy ,n,indexh,ngrid)
        ws(nn+2)=wwn(ixp,jy ,n,indexh,ngrid)
        ws(nn+3)=wwn(ix ,jyp,n,indexh,ngrid)
        ws(nn+4)=wwn(ixp,jyp,n,indexh,ngrid)
20      continue
      dxdtprof(n)=(y1(1)*dt2+y1(2)*dt1)*dtt
      dydtprof(n)=(y2(1)*dt2+y2(2)*dt1)*dtt
      dzdtprof(n)=(y3(1)*dt2+y3(2)*dt1)*dtt
      rhoprof(n)=(rho1(1)*dt2+rho1(2)*dt1)*dtt
      rhogradprof(n)=(rhograd1(1)*dt2+rhograd1(2)*dt1)*dtt
      indzindicator(n)=.false.


C Compute standard deviations
*****************************

      call sigma(us,usigprof(n),8)
      call sigma(vs,vsigprof(n),8)
      call sigma(ws,wsigprof(n),8)


      end
