      subroutine interpol_wind(itime,xt,yt,zt,dxdt,dydt,
     +dzdt,usig,vsig,wsig,pvi)
C                                i   i  i  i   o    o
C      o    o    o    o    o
********************************************************************************
*                                                                              *
*  This subroutine interpolates the wind data to current trajectory position.  *
*                                                                              *
*    Author: A. Stohl                                                          *
*                                                                              *
*    16 December 1997                                                          *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* dxdt,dydt,dzdt     wind components in grid units per second                  *
* itime [s]          current temporal position                                 *
* memtime(3) [s]     times of the wind fields in memory                        *
* xt,yt,zt           coordinates position for which wind data shall be calculat*
*                                                                              *
* Constants:                                                                   *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'
      include 'includeinterpol'

      integer itime
      real xt,yt,zt,dxdt,dydt,dzdt,usig,vsig,wsig,pvi

C Auxiliary variables needed for interpolation
      real dz1,dz2,dz
      real u1(2),v1(2),w1(2),pv1(2),uh(2),vh(2),wh(2),pvh(2)
      real us(16),vs(16),ws(16)
      integer i,m,n,nn,indexh,indzh


*********************************************
C Multilinear interpolation in time and space
*********************************************

C Determine the lower left corner and its distance to the current position
**************************************************************************

      ddx=xt-float(ix)                     
      ddy=yt-float(jy)
      rddx=1.-ddx
      rddy=1.-ddy
      p1=rddx*rddy
      p2=ddx*rddy
      p3=rddx*ddy
      p4=ddx*ddy

C Calculate variables for time interpolation
********************************************

      dt1=float(itime-memtime(1))
      dt2=float(memtime(2)-itime)
      dtt=1./(dt1+dt2)

C Determine the level below the current position for u,v
********************************************************

      do 5 i=2,nz
        if (height(i).gt.zt) then
          indz=i-1
          goto 6
        endif
5       continue
6     continue


C Vertical distance to the level below and above current position
*****************************************************************

      dz1=zt-height(indz)
      dz2=height(indz+1)-zt
      dz=1./(dz1+dz2)

***********************************************************************
C 1.) Bilinear horizontal interpolation
C This has to be done separately for 6 fields (Temporal(2)*Vertical(3))
***********************************************************************

C Loop over 2 time steps and 2 levels
*************************************

      do 10 m=1,2
        indexh=memind(m)
        do 20 n=1,2
          indzh=indz+n-1
          nn=(m-1)*8+(n-1)*4

          if (ngrid.lt.0) then
            u1(n)=p1*uupol(ix ,jy ,indzh,indexh)
     +           +p2*uupol(ixp,jy ,indzh,indexh)
     +           +p3*uupol(ix ,jyp,indzh,indexh)
     +           +p4*uupol(ixp,jyp,indzh,indexh)
            v1(n)=p1*vvpol(ix ,jy ,indzh,indexh)
     +           +p2*vvpol(ixp,jy ,indzh,indexh)
     +           +p3*vvpol(ix ,jyp,indzh,indexh)
     +           +p4*vvpol(ixp,jyp,indzh,indexh)

            us(nn+1)=uupol(ix ,jy ,indzh,indexh)
            us(nn+2)=uupol(ixp,jy ,indzh,indexh)
            us(nn+3)=uupol(ix ,jyp,indzh,indexh)
            us(nn+4)=uupol(ixp,jyp,indzh,indexh)

            vs(nn+1)=vvpol(ix ,jy ,indzh,indexh)
            vs(nn+2)=vvpol(ixp,jy ,indzh,indexh)
            vs(nn+3)=vvpol(ix ,jyp,indzh,indexh)
            vs(nn+4)=vvpol(ixp,jyp,indzh,indexh)
          else
            u1(n)=p1*uu(ix ,jy ,indzh,indexh)
     +           +p2*uu(ixp,jy ,indzh,indexh)
     +           +p3*uu(ix ,jyp,indzh,indexh)
     +           +p4*uu(ixp,jyp,indzh,indexh)
            v1(n)=p1*vv(ix ,jy ,indzh,indexh)
     +           +p2*vv(ixp,jy ,indzh,indexh)
     +           +p3*vv(ix ,jyp,indzh,indexh)
     +           +p4*vv(ixp,jyp,indzh,indexh)

            us(nn+1)=uu(ix ,jy ,indzh,indexh)
            us(nn+2)=uu(ixp,jy ,indzh,indexh)
            us(nn+3)=uu(ix ,jyp,indzh,indexh)
            us(nn+4)=uu(ixp,jyp,indzh,indexh)

            vs(nn+1)=vv(ix ,jy ,indzh,indexh)
            vs(nn+2)=vv(ixp,jy ,indzh,indexh)
            vs(nn+3)=vv(ix ,jyp,indzh,indexh)
            vs(nn+4)=vv(ixp,jyp,indzh,indexh)
          endif
          w1(n)=p1*ww(ix ,jy ,indzh,indexh)
     +         +p2*ww(ixp,jy ,indzh,indexh)
     +         +p3*ww(ix ,jyp,indzh,indexh)
     +         +p4*ww(ixp,jyp,indzh,indexh)

          ws(nn+1)=ww(ix ,jy ,indzh,indexh)
          ws(nn+2)=ww(ixp,jy ,indzh,indexh)
          ws(nn+3)=ww(ix ,jyp,indzh,indexh)
          ws(nn+4)=ww(ixp,jyp,indzh,indexh)

          pv1(n)=p1*pv(ix ,jy ,indzh,indexh)
     +          +p2*pv(ixp,jy ,indzh,indexh)
     +          +p3*pv(ix ,jyp,indzh,indexh)
     +          +p4*pv(ixp,jyp,indzh,indexh)
20        continue


***********************************
C 2.) Linear vertical interpolation
***********************************

        uh(m)=(dz2*u1(1)+dz1*u1(2))*dz
        vh(m)=(dz2*v1(1)+dz1*v1(2))*dz
        wh(m)=(dz2*w1(1)+dz1*w1(2))*dz
10      pvh(m)=(dz2*pv1(1)+dz1*pv1(2))*dz



*************************************
C 3.) Temporal interpolation (linear)
*************************************

      dxdt=(uh(1)*dt2+uh(2)*dt1)*dtt
      dydt=(vh(1)*dt2+vh(2)*dt1)*dtt
      dzdt=(wh(1)*dt2+wh(2)*dt1)*dtt
      pvi=(pvh(1)*dt2+pvh(2)*dt1)*dtt


C Compute standard deviations
*****************************

      call sigma(us,usig,16)
      call sigma(vs,vsig,16)
      call sigma(ws,wsig,16)


      end
