      subroutine interpol_wind_short_nests(itime,xt,yt,zt,dxdt,dydt,
     +dzdt)
C                                            i   i  i  i   o    o
C      o
********************************************************************************
*                                                                              *
*  This subroutine interpolates the wind data to current trajectory position.  *
*                                                                              *
*    Author: A. Stohl                                                          *
*                                                                              *
*    16 December 1997                                                          *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* dxdt,dydt,dzdt     wind components in grid units per second                  *
* itime [s]          current temporal position                                 *
* memtime(3) [s]     times of the wind fields in memory                        *
* xt,yt,zt           coordinates position for which wind data shall be calculat*
*                                                                              *
* Constants:                                                                   *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'
      include 'includeinterpol'

      integer itime
      real xt,yt,zt,dxdt,dydt,dzdt

C Auxiliary variables needed for interpolation
      real dz1,dz2,dz
      real u1(2),v1(2),w1(2),uh(2),vh(2),wh(2)
      integer i,m,n,indexh,indzh


*********************************************
C Multilinear interpolation in time and space
*********************************************

      ddx=xt-float(ix)                     
      ddy=yt-float(jy)
      rddx=1.-ddx
      rddy=1.-ddy
      p1=rddx*rddy
      p2=ddx*rddy
      p3=rddx*ddy
      p4=ddx*ddy

C Calculate variables for time interpolation
********************************************

      dt1=float(itime-memtime(1))
      dt2=float(memtime(2)-itime)
      dtt=1./(dt1+dt2)

C Determine the level below the current position for u,v
********************************************************

      do 5 i=2,nz
        if (height(i).gt.zt) then
          indz=i-1
          goto 6
        endif
5       continue
6     continue


C Vertical distance to the level below and above current position
*****************************************************************

      dz1=zt-height(indz)
      dz2=height(indz+1)-zt
      dz=1./(dz1+dz2)

***********************************************************************
C 1.) Bilinear horizontal interpolation
C This has to be done separately for 6 fields (Temporal(2)*Vertical(3))
***********************************************************************

C Loop over 2 time steps and 2 levels
*************************************

      do 10 m=1,2
        indexh=memind(m)
        do 20 n=1,2
          indzh=indz+n-1

          u1(n)=p1*uun(ix ,jy ,indzh,indexh,ngrid)
     +         +p2*uun(ixp,jy ,indzh,indexh,ngrid)
     +         +p3*uun(ix ,jyp,indzh,indexh,ngrid)
     +         +p4*uun(ixp,jyp,indzh,indexh,ngrid)
          v1(n)=p1*vvn(ix ,jy ,indzh,indexh,ngrid)
     +         +p2*vvn(ixp,jy ,indzh,indexh,ngrid)
     +         +p3*vvn(ix ,jyp,indzh,indexh,ngrid)
     +         +p4*vvn(ixp,jyp,indzh,indexh,ngrid)
          w1(n)=p1*wwn(ix ,jy ,indzh,indexh,ngrid)
     +         +p2*wwn(ixp,jy ,indzh,indexh,ngrid)
     +         +p3*wwn(ix ,jyp,indzh,indexh,ngrid)
     +         +p4*wwn(ixp,jyp,indzh,indexh,ngrid)

20        continue


***********************************
C 2.) Linear vertical interpolation
***********************************

        uh(m)=(dz2*u1(1)+dz1*u1(2))*dz
        vh(m)=(dz2*v1(1)+dz1*v1(2))*dz
10      wh(m)=(dz2*w1(1)+dz1*w1(2))*dz



*************************************
C 3.) Temporal interpolation (linear)
*************************************

      dxdt=(uh(1)*dt2+uh(2)*dt1)*dtt
      dydt=(vh(1)*dt2+vh(2)*dt1)*dtt
      dzdt=(wh(1)*dt2+wh(2)*dt1)*dtt

      end
