      subroutine readoutgrid()
********************************************************************************
*                                                                              *
*     This routine reads the user specifications for the output grid.          *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     4 June 1996                                                              *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* dxout,dyout          grid distance                                           *
* numxgrid,numygrid,numzgrid    grid dimensions                                *
* outlon0,outlat0      lower left corner of grid                               *
* outheight(maxzgrid)  height levels of output grid [m]                        *
*                                                                              *
* Constants:                                                                   *
* unitoutgrid          unit connected to file OUTGRID                          *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'

      integer i,j
      real outhelp,xr,xr1,yr,yr1,eps
      parameter(eps=1.e-4)



C Open the OUTGRID file and read output grid specifications
***********************************************************

      open(unitoutgrid,file=path(1)(1:len(1))//'OUTGRID',status='old',
     +err=999)


      call skplin(5,unitoutgrid)


C 1.  Read horizontal grid specifications
*****************************************

      call skplin(3,unitoutgrid)
      read(unitoutgrid,'(4x,f11.4)') outlon0
      call skplin(3,unitoutgrid)
      read(unitoutgrid,'(4x,f11.4)') outlat0
      call skplin(3,unitoutgrid)
      read(unitoutgrid,'(4x,i5)') numxgrid
      call skplin(3,unitoutgrid)
      read(unitoutgrid,'(4x,i5)') numygrid
      call skplin(3,unitoutgrid)
      read(unitoutgrid,'(4x,f12.5)') dxout
      call skplin(3,unitoutgrid)
      read(unitoutgrid,'(4x,f12.5)') dyout


C Check validity of output grid (shall be within model domain)
**************************************************************

      xr=outlon0+float(numxgrid)*dxout
      yr=outlat0+float(numygrid)*dyout
      xr1=xlon0+float(nx-1)*dx
      yr1=ylat0+float(ny-1)*dy
      if ((outlon0+eps.lt.xlon0).or.(outlat0+eps.lt.ylat0)
     +.or.(xr.gt.xr1+eps).or.(yr.gt.yr1+eps)) then
        write(*,*) ' #### FLEXPART MODEL ERROR! PART OF OUTPUT    ####'
        write(*,*) ' #### GRID IS OUTSIDE MODEL DOMAIN. CHANGE    ####'
        write(*,*) ' #### FILE OUTGRID IN DIRECTORY               ####'
        write(*,'(a)') path(1)(1:len(1))
        stop
      endif
      if ((numxgrid.gt.maxxgrid).or.(numygrid.gt.maxygrid)) then
        write(*,*) ' #### FLEXPART MODEL ERROR! DIMENSIONS OF     ####'
        write(*,*) ' #### OUTPUT GRID EXCEED MAXIMUM VALUES.      ####'
        write(*,*) ' #### CHANGE FILE $FLEXPART/options/OUTGRID.  ####'
        stop
      endif

C 2. Vertical levels of output grid
***********************************

      j=0
100     j=j+1
        do 20 i=1,3
20        read(unitoutgrid,*,end=99)
        read(unitoutgrid,'(4x,f7.1)',end=99) outhelp
        if (outhelp.eq.0.) goto 99
        if (j.gt.maxzgrid) then
        write(*,*) ' #### FLEXPART MODEL ERROR! TOO MANY HEIGHT   #### ' 
        write(*,*) ' #### LEVELS ARE GIVEN FOR OUTPUT GRID.       #### ' 
        write(*,*) ' #### MAXIMUM NUMBER IS ',maxzgrid,'          #### ' 
        write(*,*) ' #### PLEASE MAKE CHANGES IN FILE OUTGRID.    #### ' 
        endif
        outheight(j)=outhelp
        goto 100
99    numzgrid=j-1


C Check whether vertical levels are specified in ascending order
****************************************************************

      do 40 j=2,numzgrid
        if (outheight(j).le.outheight(j-1)) then
        write(*,*) ' #### FLEXPART MODEL ERROR! YOUR SPECIFICATION#### ' 
        write(*,*) ' #### OF OUTPUT LEVELS IS CORRUPT AT LEVEL    #### ' 
        write(*,*) ' #### ',j,'                              #### ' 
        write(*,*) ' #### PLEASE MAKE CHANGES IN FILE OUTGRID.    #### ' 
        endif
40      continue

C Determine the half levels, i.e. middle levels of the output grid
******************************************************************

      outheighthalf(1)=outheight(1)/2.
      do 50 j=2,numzgrid
50      outheighthalf(j)=(outheight(j-1)+outheight(j))/2.


      xoutshift=xlon0-outlon0
      youtshift=ylat0-outlat0
      close(unitoutgrid)
      return


999   write(*,*) ' #### FLEXPART MODEL ERROR! FILE "OUTGRID"    #### '
      write(*,*) ' #### CANNOT BE OPENED IN THE DIRECTORY       #### '
      write(*,*) ' #### xxx/flexpart/options                    #### '
      stop

      end
