      subroutine redist (ipart,ktop,ipconv)
c**************************************************************************
C Do the redistribution of particles due to convection
C This subroutine is called for each particle which is assigned
C a new vertical position randomly, based on the convective redistribution
C matrix
c**************************************************************************

C Petra Seibert, Feb 2001, Apr 2001, May 2001, Jan 2002, Nov 2002 and
C Andreas Frank, Nov 2002

c Caroline Forster:  November 2004 - February 2005

      include 'includepar'
      include 'includecom'
      include 'includeconv'

      real const
      parameter (const=r_air/ga) 
      integer ipart, ktop,ipconv
      integer iseed, k, kz, levnew, levold
      real uvzlev(nuvzmax), uvzeta(0:nuvzmax) 
      real wsub(nuvzmax) 
      real totlevmass, wsubpart
      real pint, pold, rn, tv, tvold, dlevfrac
      real ew,ran3, ztold,fraction
      real tv1, tv2, dlogp, dz, dz1, dz2
      real pint_1,pold_1,tv_1,tvold_1
      save iseed, uvzlev
      data iseed /-88/

C ipart   ... number of particle to be treated

c     ipconv=1

c determine height of eta levels (uvzeta) and height of the
c eta half-levels (uvzlev)
c do that only once for each grid column
c i.e. when ktop.eq.1
***************************************************************

      if (ktop .le. 1) then

        tvold=tt2conv*(1.+0.378*ew(td2conv)/psconv)
        tvold_1=tvold
        pold=psconv
        pold_1=psconv
        uvzlev(1)=0.
        uvzeta(0)=0.

        pint = phconv(2)
C       determine next virtual temperatures
        tv1 = tconv(1)*(1.+0.608*qconv(1))
        tv2 = tconv(2)*(1.+0.608*qconv(2))
C       interpolate virtual temperature to half-level
        tv = tv1 + (tv2-tv1)*(pconv(1)-phconv(2))/(pconv(1)-pconv(2))
        if (abs(tv-tvold).gt.0.2) then
          uvzlev(2) = uvzlev(1) + 
     &                  const*log(pold/pint)*
     &                 (tv-tvold)/log(tv/tvold)
        else
          uvzlev(2) = uvzlev(1)+
     &          const*log(pold/pint)*tv
        endif
        tvold=tv
        tv1=tv2
        pold=pint

C integrate profile (calculation of height agl of eta layers) as required
        do 10 kz = 3, nconvtop+1
C         note that variables defined in calcmatrix.f (pconv,tconv,qconv)
C         start at the first real ECMWF model level whereas kz and
C         thus uvzlev(kz) starts at the surface. uvzlev is defined at the
C         half-levels (between the tconv, qconv etc. values !)
C         Thus, uvzlev(kz) is the lower boundary of the tconv(kz) cell.
c         while uvzeta(kz) is defined at tconv(kz) 
          pint = phconv(kz)
C         determine next virtual temperatures
          tv2 = tconv(kz)*(1.+0.608*qconv(kz))
C         interpolate virtual temperature to half-level
          tv = tv1 + (tv2-tv1)*(pconv(kz-1)-phconv(kz))/
     &         (pconv(kz-1)-pconv(kz))
          if (abs(tv-tvold).gt.0.2) then
            uvzlev(kz) = uvzlev(kz-1) + 
     &                  const*log(pold/pint)*
     &                 (tv-tvold)/log(tv/tvold)
          else
            uvzlev(kz) = uvzlev(kz-1)+
     &          const*log(pold/pint)*tv
          endif
          tvold=tv
          tv1=tv2
          pold=pint

c eta levels (uvzeta)

          k = kz-1
          pint_1=akz(k)+bkz(k)*psconv
          tv_1=tconv(k)*(1.+0.608*qconv(k))

          if (abs(tv_1-tvold_1).gt.0.2) then
            uvzeta(k-1)=uvzeta(k-2)+const*log(pold_1/pint_1)*
     +        (tv_1-tvold_1)/log(tv_1/tvold_1)
          else
            uvzeta(k-1)=uvzeta(k-2)+const*log(pold_1/pint_1)*tv_1
          endif

          tvold_1=tv_1
          pold_1=pint_1

10      continue

        ktop = 2


      endif ! (if ktop .le. 1) then

C  determine vertical grid position of particle in the eta system
C****************************************************************

      ztold = ztra1(abs(ipart))
C find old particle grid position
      do 20 kz = 2, nconvtop
        if (uvzlev(kz) .ge. ztold ) then
          levold = kz-1
          goto 30
        endif
20    continue

C Particle is above the potentially convective domain. Skip it.
      goto 90

30    continue

c now redistribute particles
c****************************

C  Choose a random number and find corresponding level of destination
C  Random numbers to be evenly distributed in [0,1]

      rn = ran3(iseed)

c initialize levnew

      levnew = levold

      fraction = 0.
      totlevmass=dp(levold)/ga
      do 35 k = 1,nconvtop
c for backward runs use the transposed matrix
       if (ldirect.eq.1) then
         fraction=fraction+fmassfrac(levold,k)
     +       /totlevmass
       else
         fraction=fraction+fmassfrac(k,levold)
     +       /totlevmass
       endif
       if (rn.le.fraction) then
         levnew=k
c avoid division by zero or a too small number
c if division by zero or a too small number happens the
c particle is assigned to the center of the grid cell
         if (fraction.gt.1.e-20) then
          dlevfrac = (fraction-rn) / fraction
         else
          dlevfrac = 0.5
         endif
         goto 40
       endif
 35   continue

40    continue

C now assign new position to particle

      if (levnew.le.nconvtop) then
       if (levnew.eq.levold) then
          ztra1(abs(ipart)) = ztold
       else
        dlogp = (1.-dlevfrac)*
     +        (log(phconv(levnew+1))-log(phconv(levnew)))
        pint = log(phconv(levnew))+dlogp
        dz1 = pint - log(phconv(levnew))
        dz2 = log(phconv(levnew+1)) - pint
        dz = dz1 + dz2
        ztra1(abs(ipart)) = (uvzlev(levnew)*dz2+uvzlev(levnew+1)*dz1)/dz
c        if (ipconv.gt.0) ipconv=-1
        endif
      endif

c displace particle according to compensating subsidence
c this is done to ALL particles, not only to those redistributed by convection
******************************************************************************

c find particle position with repect to height of the eta levels
c i.e. the levels where tconv, qconv, and sub are given
c start searching from uvzeta(levnew-1), which is one half level below uvzlev(levnew)
c do nothing with particles below uvzeta(1)

      ztold = ztra1(abs(ipart))

      if (ztold.le.uvzeta(1)) goto 90

      do kz = levnew-1, nconvtop
        if (uvzeta(kz) .ge. ztold ) then
          levold = kz-1
          goto 31
        endif
       end do
  31   continue

c determine compensating vetical velocity at the levels
c above and below the particel position

       wsub(levold)=-1.*sub(levold)*r_air*tconv(levold)/(pconv(levold))
       wsub(levold+1)=-1.*sub(levold+1)*r_air*tconv(levold+1)/
     & (pconv(levold+1))


c interpolate wsub to the vertical particle position

      dz1 = ztold - uvzeta(levold)
      dz2 = uvzeta(levold+1) - ztold
      dz = dz1 + dz2

      wsubpart = (dz2*wsub(levold)+dz1*wsub(levold+1))/dz
      ztra1(abs(ipart)) = ztold+wsubpart*float(lsynctime)
      if (ztra1(abs(ipart)).lt.0.) then
         ztra1(abs(ipart))=-1.*ztra1(abs(ipart))
      endif

C Maximum altitude .5 meter below uppermost model level
********************************************************

 90   continue

      if (ztra1(abs(ipart)) .gt. height(nz)-0.5)
     &    ztra1(abs(ipart)) = height(nz)-0.5

      end
